import json
from tkinter import EXCEPTION
from .constants import *
from .logging import Log
from .exceptions import *
from .utils import *
import os
from .login import *
from time import sleep
from datetime import datetime, date,timedelta
import logging
from playwright.async_api import async_playwright



class Upload:
    def __init__(
        self,
        root_profile_directory: str,
        proxy_option: str = "",
        timeout: int = 3,
        watcheveryuploadstep: bool = True,
        debug: bool = True,
        username:str ="",
        password:str ="",
        CHANNEL_COOKIES: str = "",
        ytb_cookies:str="",
        tiktok_cookies:str="",
        recordvideo:bool=False

    ) -> None:
        self.timeout = timeout
        self.log = Log(debug)
        self.username=username
        self.password=password
        self.CHANNEL_COOKIES = CHANNEL_COOKIES    
        self.root_profile_directory=root_profile_directory
        self.proxy_option=proxy_option
        self.watcheveryuploadstep=watcheveryuploadstep
        self.ytb_cookies=ytb_cookies
        self.tiktok_cookies=tiktok_cookies
        self._playwright=''
        self.browser=''
        self.context=''
        self.page=''
        self.recordvideo=recordvideo
        # self.setup()

    def send(self, element, text: str) -> None:
        element.clear()
        sleep(self.timeout)
        element.send_keys(text)
        sleep(self.timeout)

    async def click_next(self, page) -> None:
        await page.locator(NEXT_BUTTON).click()
        sleep(self.timeout)

    async def not_uploaded(self, page) -> bool:
        s=await page.locator(STATUS_CONTAINER).text_content()
        return s.find(UPLOADED) != -1

    async def upload(
        self,
        file: str,
        title: str = "",
        description: str = "",
        thumbnail: str = "",
        publishpolicy: str = 0,
        # mode a:release_offset exist,publish_data exist will take date value as a starting date to schedule videos
        # mode b:release_offset not exist, publishdate exist , schedule to this specific date
        # mode c:release_offset not exist, publishdate not exist,daily count to increment schedule from tomorrow
        # mode d: offset exist, publish date not exist, daily count to increment with specific offset schedule from tomorrow
        release_offset: str = '0-1',
        publish_date: datetime = datetime(
            date.today().year,  date.today().month,  date.today().day, 10, 15),
        tags: list = [],
        closewhen100percentupload:bool =True
    ) -> Tuple[bool, Optional[str]]:
        """Uploads a video to YouTube.
        Returns if the video was uploaded and the video id.
        """
        self._playwright = await self._start_playwright()
            #     browser = p.chromium.launch()

        # proxy_option = "socks5://127.0.0.1:1080"

        headless=True
        if self.watcheveryuploadstep:
            headless=False
        print('whether run in view mode',headless)
        if self.proxy_option == "":
            print('start web page without proxy')

            browserLaunchOptionDict = {
                "headless": headless,
                # "executable_path": executable_path,
                "timeout": 30000
            }

            if not self.root_profile_directory:
                self.browser = await self._start_browser("firefox", **browserLaunchOptionDict)
            else:
                self.browser = await self._start_persistent_browser(
                    "firefox", user_data_dir=self.root_profile_directory, **browserLaunchOptionDict
                )
            if self.recordvideo:
                self.context = await self.browser.new_context(record_video_dir="test-results")
            else:
                self.context = await self.browser.new_context()
        else:
            print('start web page with proxy')

            browserLaunchOptionDict = {
                "headless": headless,
                "proxy": {
                    "server": self.proxy_option,
                },

                # timeout <float> Maximum time in milliseconds to wait for the browser instance to start. Defaults to 30000 (30 seconds). Pass 0 to disable timeout.#
                "timeout": 30000
            }

            if not self.root_profile_directory:
                self.browser = await self._start_browser("firefox", **browserLaunchOptionDict)
            else:
                self.browser = await self._start_persistent_browser(
                    "firefox", user_data_dir=self.root_profile_directory, **browserLaunchOptionDict
                )
        # Open new page
            if self.recordvideo:
                self.context = await self.browser.new_context(record_video_dir="test-results")
            else:
                self.context = await self.browser.new_context()
        self.log.debug("Firefox is now running")
        page = await self.context.new_page()
        print('============tags',tags)
        if not file:
            raise FileNotFoundError(f'Could not find file with path: "{file}"')


        if self.CHANNEL_COOKIES and not self.CHANNEL_COOKIES == '':
            print('cookies existing', self.CHANNEL_COOKIES)

            await self.context.clear_cookies()

            await self.context.add_cookies(
                json.load(
                    open(
                        self.CHANNEL_COOKIES, 
                        'r'
                    )
                )
            )            
            # login_using_cookie_file(self,self.CHANNEL_COOKIES,page)         
            await page.goto(YOUTUBE_URL,timeout=300000)

            await page.reload()
        else:
            self.log.info('Please sign in and then press enter')
            # input()

            await page.goto(YOUTUBE_URL,timeout=300000)
            # Interact with login form
            browser_context = await self.browser.new_context(
                ignore_https_errors=True)
            await browser_context.clear_cookies()
            # page.click('text=Login')
            # page.fill('input[name="login"]', USERNAME)
            # page.fill('input[name="password"]', PASSWORD)
            # page.click('text=Submit')
            sleep(USER_WAITING_TIME)
            storage = await browser_context.storage_state(path=self.CHANNEL_COOKIES)
            self.context = browser_context

        islogin = confirm_logged_in(page)
        print('checking login status', islogin)

        if not islogin:
            print('try to load cookie files')
            await self.context.clear_cookies()

            await self.context.add_cookies(
                json.load(
                    open(
                        self.CHANNEL_COOKIES, 
                        'r'
                    )
                )
            )            

            print('success load cookie files')
            await page.goto(YOUTUBE_URL,timeout=300000)
            print('start to check login status')

            islogin = confirm_logged_in(page)

            # https://github.com/xtekky/google-login-bypass/blob/main/login.py

        print('start change locale to english')

        await set_channel_language_english(page)
        print('finish change locale to english')
        await page.goto(YOUTUBE_UPLOAD_URL,timeout=300000)
        # sleep(self.timeout)
        self.log.debug("Found YouTube upload Dialog Modal")

        self.log.debug(f'Trying to upload "{file}" to YouTube...')
        if os.path.exists(get_path(file)):
            page.locator(
                INPUT_FILE_VIDEO)
            await page.set_input_files(INPUT_FILE_VIDEO, get_path(file))
        else:
            if os.path.exists(file.encode('utf-8')):
                print('file found', file)
                page.locator(
                    INPUT_FILE_VIDEO)
                await page.set_input_files(INPUT_FILE_VIDEO, file.encode('utf-8'))
        sleep(self.timeout)
        textbox=page.locator(TEXTBOX)
        # accountcheck=await textbox.is_editable()
        # if not accountcheck:

# fix google account verify
        try:

            while True:
                check = page.locator('//*[@id="dialog-title"]')
                self.log.debug(f'found to YouTube account check')
                # sleep(60)
                # await verify(self,page)
                # await page.goto(YOUTUBE_UPLOAD_URL)
                x_path = '//*[@id="textbox"]'
                if page.locator(x_path):
                    self.log.debug(f'fix  YouTube account check')
                    break

        except:
            sleep(1)

        #confirm-button > div:nth-child(2)
        # # Catch max uploads/day limit errors
        # if page.get_attribute(NEXT_BUTTON, 'hidden') == 'true':
        #     error_short_by_xpath=page.locator(ERROR_SHORT_XPATH)
        #     # print(f"ERROR: {error_short_by_xpath.text} {self.cookie_working_dir}")
        #     return False

        self.log.debug(f'Trying to set "{title}" as title...')


        # get file name (default) title
        # title=title if title else page.locator(TEXTBOX).text_content()
        # print(title)
        sleep(self.timeout)
        if len(title) > TITLE_COUNTER:
            print(f"Title was not set due to exceeding the maximum allowed characters ({len(title)}/{TITLE_COUNTER})")
            title=title[:TITLE_COUNTER-1]

                # TITLE
        print('click title field to input')
        titlecontainer= page.locator(TEXTBOX)
        await titlecontainer.click()
        print('clear existing title')
        await page.keyboard.press("Backspace")
        await page.keyboard.press("Control+KeyA")
        await page.keyboard.press("Delete")
        print('filling new  title')

        await page.keyboard.type(title)

        self.log.debug(f'Trying to set "{title}" as description...')

        if description:
            if len(description) > DESCRIPTION_COUNTER:
                print(
                    f"Description was not set due to exceeding the maximum allowed characters ({len(description)}/{DESCRIPTION_COUNTER})"
                )
                description=description[:4888]

            self.log.debug(f'Trying to set "{description}" as description...')
            print('click description field to input')
            await page.locator(DESCRIPTION_CONTAINER).click()
            print('clear existing description')
            await page.keyboard.press("Backspace")
            await page.keyboard.press("Control+KeyA")
            await page.keyboard.press("Delete")
            print('filling new  description')

            await page.keyboard.type(description)


        if thumbnail:
            self.log.debug(f'Trying to set "{thumbnail}" as thumbnail...')
            if os.path.exists(get_path(thumbnail)):
                await page.locator(
                    INPUT_FILE_THUMBNAIL).set_input_files(get_path(thumbnail))
            else:
                if os.path.exists(thumbnail.encode('utf-8')):
                    print('thumbnail found', thumbnail)
                    await page.locator(INPUT_FILE_THUMBNAIL).set_input_files(
                        thumbnail.encode('utf-8'))
            sleep(self.timeout)

        self.log.debug('Trying to set video to "Not made for kids"...')
        
        kids_section=page.locator(NOT_MADE_FOR_KIDS_LABEL)
        await page.locator(NOT_MADE_FOR_KIDS_RADIO_LABEL).click()
        sleep(self.timeout)
        print('not made for kids done')
        if tags is None or tags =="" or len(tags)==0:
            pass
        else:
            print('tags you give',tags)
            if type(tags) == list:
                tags=",".join(str(tag) for tag in tags)
                tags=tags[:500]
            else:
                tags=tags
            print('overwrite prefined channel tags',tags)
            if len(tags) > TAGS_COUNTER:
                print(f"Tags were not set due to exceeding the maximum allowed characters ({len(tags)}/{TAGS_COUNTER})")
                tags=tags[:TAGS_COUNTER]
            print('click show more button')
            sleep(self.timeout)
            await page.locator(MORE_OPTIONS_CONTAINER).click()

            self.log.debug(f'Trying to set "{tags}" as tags...')
            await page.locator(TAGS_CONTAINER).locator(TEXT_INPUT).click()
            print('clear existing tags')
            await page.keyboard.press("Backspace")
            await page.keyboard.press("Control+KeyA")
            await page.keyboard.press("Delete")
            print('filling new  tags')
            await page.keyboard.type(tags)

# Language and captions certification
# Recording date and location
# Shorts sampling
# Category
        if closewhen100percentupload==False:
            pass
        else:
            await wait_for_processing(page,process=False)
        # if "complete" in page.locator(".progress-label").text_content():

        # sometimes you have 4 tabs instead of 3
        # this handles both cases
        for _ in range(3):
            try:
                await self.click_next(page)
                print('next next!')
            except:
                pass
        if not int(publishpolicy) in [0, 1, 2]:
            publishpolicy=0
        if int(publishpolicy) == 0:
            self.log.debug("Trying to set video visibility to private...")

            public_main_button=page.locator(PRIVATE_BUTTON)
            await page.locator(PRIVATE_RADIO_LABEL).click()
        elif int(publishpolicy) == 1:
            self.log.debug("Trying to set video visibility to public...")

            public_main_button=page.locator(PUBLIC_BUTTON)
            await page.locator(PUBLIC_RADIO_LABEL).click()
        else:
        # mode a:release_offset exist,publish_data exist will take date value as a starting date to schedule videos
        # mode b:release_offset not exist, publishdate exist , schedule to this specific date
        # mode c:release_offset not exist, publishdate not exist,daily count to increment schedule from tomorrow
        # mode d: offset exist, publish date not exist, daily count to increment with specific offset schedule from tomorrow            
            self.log.debug(
                "Trying to set video schedule time...{publish_date}")
            if release_offset and not release_offset == "":
                    print('mode a sta')
                    if not int(release_offset.split('-')[0]) == 0:
                        offset = timedelta(months=int(release_offset.split(
                            '-')[0]), days=int(release_offset.split('-')[-1]))
                    else:
                        offset = timedelta(days=1)
                    if publish_date is None:
                        publish_date =datetime(
                            date.today().year,  date.today().month,  date.today().day, 10, 15)
                    else:
                        publish_date += offset
                
            else:
                if publish_date is None:
                    publish_date =datetime(
                        date.today().year,  date.today().month,  date.today().day, 10, 15)
                    offset = timedelta(days=1)  
                else:
                    publish_date = publish_date
                # dailycount=4

                # release_offset=str(int(start_index/30))+'-'+str(int(start_index)/int(setting['dailycount']))
                

            await setscheduletime(page,publish_date)
            # set_time_cssSelector(page,publish_date)

        video_id=await self.get_video_id(page)
        # option 1 to check final upload status
        while await self.not_uploaded(page):
            self.log.debug("Still uploading...")
            sleep(5)

        done_button=page.locator(DONE_BUTTON)

        if await done_button.get_attribute("aria-disabled") == "true":
            error_message= await page.locator(
                ERROR_CONTAINER).text_content()
            return False, error_message

        await done_button.click()
        print('upload process is done')

        # # option 2 to check final upload status

        # # Go back to endcard settings
        # page.wait_for_selector("#step-badge-1").click()
        # # self._set_endcard()

        # for _ in range(2):
        #     # Sometimes, the button is clickable but clicking it raises an error, so we add a "safety-sleep" here
        #     sleep(5)
        #     self.click_next(page)

        # sleep(5)
        # page.locator("done-button").click()

        # # Wait for the dialog to disappear
        sleep(5)
        logging.info("Upload is complete")
        await self.close()
        # page.locator("#close-icon-button > tp-yt-iron-icon:nth-child(1)").click()
        # print(page.expect_popup().locator("#html-body > ytcp-uploads-still-processing-dialog:nth-child(39)"))
        # page.wait_for_selector("ytcp-dialog.ytcp-uploads-still-processing-dialog > tp-yt-paper-dialog:nth-child(1)")
        # page.locator("ytcp-button.ytcp-uploads-still-processing-dialog > div:nth-child(2)").click()
        return True, video_id

    async def get_video_id(self, page) -> Optional[str]:
        video_id=None
        try:
            video_url_container=page.locator(
                VIDEO_URL_CONTAINER)
            video_url_element=video_url_container.locator(
                VIDEO_URL_ELEMENT
            )

            video_id=await video_url_element.get_attribute(HREF)
            video_id=video_id.split("/")[-1]
        except:
            raise VideoIDError("Could not get video ID")

        return video_id

    # @staticmethod
    async  def _start_playwright(self):
        #  sync_playwright().start()
        return await  async_playwright().start()
    async def _start_browser(self, browser: str, **kwargs):
        if browser == "chromium":
            return await self._playwright.chromium.launch(**kwargs)

        if browser == "firefox":
            return await self._playwright.firefox.launch(**kwargs)

        if browser == "webkit":
            return await self._playwright.webkit.launch(**kwargs)

        raise RuntimeError(
            "You have to select either 'chromium', 'firefox', or 'webkit' as browser."
        )

    async def _start_persistent_browser(
        self, browser: str, user_data_dir: Optional[Union[str, Path]], **kwargs
    ):
        if browser == "chromium":
            return await self._playwright.chromium.launch_persistent_context(
                user_data_dir, **kwargs
            )
        if browser == "firefox":
            return await self._playwright.firefox.launch_persistent_context(
                user_data_dir, **kwargs
            )
        if browser == "webkit":
            return await self._playwright.webkit.launch_persistent_context(
                user_data_dir, **kwargs
            )

        raise RuntimeError(
            "You have to select either 'chromium', 'firefox' or 'webkit' as browser."
        )
    async def close(self):
        await self.browser.close()
        await self._playwright.stop()


# URLS
TIKTOK_URL= f"https://www.tiktok.com/upload?lang=en"

YOUTUBE_URL = "https://www.youtube.com"
YOUTUBE_STUDIO_URL = "https://studio.youtube.com"
YOUTUBE_UPLOAD_URL = "https://www.youtube.com/upload"
USER_WAITING_TIME = 1

# CONTAINERS
CONFIRM_CONTAINER='#confirmation-dialog'
TAGS_CONTAINER = '//*[@id="tags-container"]'
ERROR_CONTAINER = '//*[@id="error-message"]'
STATUS_CONTAINER = "//html/body/ytcp-uploads-dialog/tp-yt-paper-dialog/div/ytcp-animatable[2]/div/div[1]/ytcp-video-upload-progress/span"
VIDEO_URL_CONTAINER = "//span[@class='video-url-fadeable style-scope ytcp-video-info']"
DESCRIPTION_CONTAINER = "//*[@id='description-container']"
MORE_OPTIONS_CONTAINER = "#toggle-button > div:nth-child(2)"
TIME_BETWEEN_POSTS = 3600
# COUNTERS
TAGS_COUNTER = 500
TITLE_COUNTER = 100
DESCRIPTION_COUNTER = 5000

# OTHER
HREF = "href"
TEXTBOX = "#title-textarea"
UPLOADED = "Uploading"
TEXT_INPUT = "#text-input"
NOT_MADE_FOR_KIDS_RADIO_LABEL = "tp-yt-paper-radio-button.ytkc-made-for-kids-select:nth-child(2) > div:nth-child(2) > ytcp-ve:nth-child(1)"
DONE_BUTTON = "//*[@id='done-button']"
NEXT_BUTTON = "//*[@id='next-button']"
PUBLIC_RADIO_LABEL ="tp-yt-paper-radio-button.style-scope:nth-child(20)"
PRIVATE_RADIO_LABEL ="#private-radio-button > div:nth-child(1)"


PUBLIC_BUTTON = "PUBLIC"
PRIVATE_BUTTON = "PRIVATE"
RADIO_CONTAINER = "//*[@id='radioContainer']"
PUBLISH_DATE="//html/body/ytcp-uploads-dialog/tp-yt-paper-dialog/div/ytcp-animatable[1]/ytcp-uploads-review/div[2]/div[1]/ytcp-video-visibility-select/div[2]/ytcp-visibility-scheduler/div[1]/ytcp-datetime-picker/div/ytcp-text-dropdown-trigger[1]/ytcp-dropdown-trigger/div/div[2]/span"
INPUT_FILE_VIDEO = "//input[@type='file']"
VIDEO_URL_ELEMENT = "//a[@class='style-scope ytcp-video-info']"
UPLOAD_DIALOG_MODAL = "#dialog.ytcp-uploads-dialog"
INPUT_FILE_THUMBNAIL = "//input[@accept='image/jpeg,image/png']"
VIDEO_NOT_FOUND_ERROR = "Could not find video_id"
NOT_MADE_FOR_KIDS_LABEL = ".made-for-kids-rating-container"
ERROR_SHORT_SELECTOR = '#dialog > div > ytcp-animatable.button-area.metadata-fade-in-section.style-scope.ytcp-uploads-dialog > div > div.left-button-area.style-scope.ytcp-uploads-dialog > div > div.error-short.style-scope.ytcp-uploads-dialog'
ERROR_SHORT_XPATH = '//*[@id="dialog"]/div/ytcp-animatable[2]/div/div[1]/div/div[1]'
UPLOADING_PROGRESS_SELECTOR = '#dialog > div > ytcp-animatable.button-area.metadata-fade-in-section.style-scope.ytcp-uploads-dialog > div > div.left-button-area.style-scope.ytcp-uploads-dialog > ytcp-video-upload-progress > span'



import time
import re
import logging
from time import sleep
from datetime import datetime
from playwright.async_api import *
from .constants import *
from pathlib import Path
import os
from typing import Tuple, Optional,Union


def get_path(file_path: str) -> str:
    # no clue why, but this character gets added for me when running
    # return str(os.path(file_path)).replace("\u202a", "")
    # return file_path.replace("\u202a", "")
    return str(Path(file_path)).replace("\u202a", "")

def close_browser(self):
    self.browser.close()
    self._playwright.stop()
async def set_channel_language_english(page):
    # why does not work again
    try:
        print('Click your profile icon .')
        page.locator(
            "yt-img-shadow.ytd-topbar-menu-button-renderer > img:nth-child(1)")
        await page.click(
            "yt-img-shadow.ytd-topbar-menu-button-renderer > img:nth-child(1)")
        print(' Click Language or Location icon')
        page.locator("yt-multi-page-menu-section-renderer.style-scope:nth-child(2) > div:nth-child(2) > ytd-compact-link-renderer:nth-child(2) > a:nth-child(1) > tp-yt-paper-item:nth-child(1) > div:nth-child(2) > yt-formatted-string:nth-child(2)")
        await page.click("yt-multi-page-menu-section-renderer.style-scope:nth-child(2) > div:nth-child(2) > ytd-compact-link-renderer:nth-child(2) > a:nth-child(1) > tp-yt-paper-item:nth-child(1) > div:nth-child(2) > yt-formatted-string:nth-child(2)")
        selector_en_path = "ytd-compact-link-renderer.style-scope:nth-child(13) > a:nth-child(1) > tp-yt-paper-item:nth-child(1) > div:nth-child(2) > yt-formatted-string:nth-child(1)"
        print('choose the language or location you like to use.')
        selector_en=page.locator(selector_en_path)
        await selector_en.click()
        # page.click(selector_en)

        # print(page.text_content('//*[@id="label"]'))
        # if page.text_content(selector_en)=="English (US)":
        #     return True
        # else:
        #     return False
        return True
    except TimeoutError:
        return False
# fix google account verify


async def verify(self, page):
    try:

        while True:
            await page.locator('#confirm-button > div:nth-child(2)').click()
            await page.goto("https://accounts.google.com/signin/v2/identifier?service=youtube&uilel=3&continue=https%3A%2F%2Fwww.youtube.com%2Fsignin%3Faction_handle_signin%3Dtrue%26app%3Ddesktop%26next%3Dhttps%253A%252F%252Fstudio.youtube.com%252Freauth%26feature%3Dreauth%26authuser%3D2%26skip_identity_prompt%3Dtrue&hl=en&authuser=2&rart=ANgoxcfF1TrrQp5lP5ySTmlJmdnwuMbSDi81WlN2aDXRgvpTnD1cv0nXHlRcMz6yv6hnqfERyjXMCgJqa8thKIAqVqatu9kTtA&flowName=GlifWebSignIn&flowEntry=ServiceLogin")
            await page.locator("#identifierId").click()
            await page.fill("#identifierId", self.username)
            await page.locator(
                ".VfPpkd-LgbsSe-OWXEXe-k8QpJ > span:nth-child(4)").click()
            time.sleep(3)
            await page.fill(
                "#password > div:nth-child(1) > div:nth-child(1) > div:nth-child(1) > input:nth-child(1)", self.password)
            await page.locator(
                ".VfPpkd-LgbsSe-OWXEXe-k8QpJ > span:nth-child(4)").click()
            time.sleep(60)

    except:
        time.sleep(1)
    # x_path = '//*[@id="textbox"]'
    # if page.wait_for_selector(x_path):
        # break        


async def wait_for_processing(page, process):
    page = page
    if process == True:
        # Wait for processing to complete
        progress_label = await page.wait_for_selector(
            "span.progress-label")
        pattern = re.compile(
            r"(finished processing)|(processing hd.*)|(check.*)")
        current_progress = await progress_label.get_attribute("textContent")
        last_progress = None
        while not pattern.match(current_progress.lower()):
            if last_progress != current_progress:
                logging.info(f'Current progress: {current_progress}')
            last_progress = current_progress
            sleep(5)
            current_progress = await progress_label.get_attribute("textContent")
    else:
        while True:

            x_path = "//span[@class='progress-label style-scope ytcp-video-upload-progress']"
# TypeError: 'WebElement' object  is not subscriptable
            upload_progress = await page.locator(
                '[class="progress-label style-scope ytcp-video-upload-progress"]').all_text_contents()

            # innerhtml = page.locator(x_path).get_attribute('innerHTML')
            # if re.match(r"\D \.\.\. \D", innerhtml) or re.match(r"^[^\.]+$", innerhtml):
            #     break
            upload_progress=' '.join(upload_progress)
            if not '%' in upload_progress.lower():
                break
            elif 'complete' in upload_progress.lower():
                break
async def setscheduletime(page, publish_date: datetime):
    hour_to_post, date_to_post, publish_date_hour=hour_and_date(
        publish_date)
    date_to_post=publish_date.strftime("%b %d, %Y")
    hour_xpath=get_hour_xpath(hour_to_post)
    # Clicking in schedule video
    print('click schedule')
    await page.locator(
        '//html/body/ytcp-uploads-dialog/tp-yt-paper-dialog/div/ytcp-animatable[1]/ytcp-uploads-review/div[2]/div[1]/ytcp-video-visibility-select/div[2]/tp-yt-paper-radio-button/div[1]/div[1]').click()
    sleep(1)
    # Writing date
    print('click date')
    await page.locator('#datepicker-trigger > ytcp-dropdown-trigger:nth-child(1) > div:nth-child(2) > div:nth-child(4)').click()

    # page.locator(
        # '//html/body/ytcp-uploads-dialog/tp-yt-paper-dialog/div/ytcp-animatable[1]/ytcp-uploads-review/div[2]/div[1]/ytcp-video-visibility-select/div[2]/ytcp-visibility-scheduler/div[1]/ytcp-datetime-picker/div/ytcp-text-dropdown-trigger[1]/ytcp-dropdown-trigger/div/div[3]').click()
    sleep(1)
    page.locator('//*[@id="input-4"]')
    
    # page.locator(
        # '//html/body/ytcp-date-picker/tp-yt-paper-dialog/div/form/tp-yt-paper-input/tp-yt-paper-input-container/div[2]/div/iron-input/input').click()
    await page.keyboard.press("Control+KeyA")
    await page.keyboard.type(date_to_post)
    await page.keyboard.press("Enter")

    sleep(1)
    print('click hour')
# #input-1
    try:
        await page.locator(
            '#input-1').click()
        sleep(1)
        await page.locator(hour_xpath).click()
    except:
        # input_hour=page.wait_for_selector(
            # 'input.tp-yt-paper-input').click()
        print('no hour input found')
    await page.keyboard.press("Control+KeyA")
    await page.keyboard.type(hour_to_post)
    await page.keyboard.press("Enter")

    sleep(1)

def hour_and_date( now_date_hour):
    # now_date_hour += datetime.timedelta(seconds=TIME_BETWEEN_POSTS)
    hour_to_post=now_date_hour.strftime('%H:%M')
    hour, minutes=hour_to_post.split(
        ':')[0], int(hour_to_post.split(':')[1])
    setting_minutes=minutes//15
    minutes=setting_minutes * 15
    if minutes == 0:
        minutes='00'
    hour_to_post=f'{hour}:{minutes}'
    date_to_post=now_date_hour.strftime('%d/%m/%Y')
    return hour_to_post, date_to_post, now_date_hour

def get_hour_xpath( input_hour):
    hour_xpath=dict()
    xpath_time=0
    for hour in range(24):
        if hour < 10 and hour >= 0:
            hour=f'0{hour}'
        for minute in range(0, 46, 15):
            if minute == 0:
                minute='00'
            xpath_time += 1
            hour_xpath.update(
                {f'{hour}:{minute}': f'//html/body/ytcp-time-of-day-picker/tp-yt-paper-dialog/tp-yt-paper-listbox/tp-yt-paper-item[{xpath_time}]'})
    return hour_xpath[input_hour]

def _set_time_cssSelector(page, publish_date: datetime):
    # Start time scheduling
    page.locator("SCHEDULE").click()

    # Open date_picker
    page.locator(
        "#datepicker-trigger > ytcp-dropdown-trigger:nth-child(1)").click()

    date_input=page.locator(
        "input.tp-yt-paper-input").click()
    date_input.clear()
    # Transform date into required format: Mar 19, 2021
    page.keyboard.press("Control+KeyA")
    page.keyboard.type(publish_date.strftime("%b %d, %Y"))
    page.keyboard.press("KeyReturn")
    # Open time_picker
    page.locator(
        "#time-of-day-trigger > ytcp-dropdown-trigger:nth-child(1) > div:nth-child(2)"
    ).click()

    time_list=page.locator(
        "tp-yt-paper-item.tp-yt-paper-item")
    # Transform time into required format: 8:15 PM
    time_str=publish_date.strftime("%I:%M %p").strip("0")

    time=[time for time in time_list[2:] if time.text == time_str][0]
    time.click()

def _set_basic_settings(page, title: str, description: str, thumbnail_path: str=None):

    title_input=page.wait_for_selector(
                '//ytcp-mention-textbox[@label="Title"]//div[@id="textbox"]'                )

    # Input meta data (title, description, etc ... )
    description_input=page.wait_for_selector(
        '//ytcp-mention-textbox[@label="Description"]//div[@id="textbox"]'
    )
    thumbnail_input=page.wait_for_selector(
        "input#file-loader"
    )

    title_input.clear()
    title_input.send_keys(title)
    description_input.send_keys(description)
    if thumbnail_path:
        thumbnail_input.send_keys(thumbnail_path)

def _set_advanced_settings(page, game_title: str, made_for_kids: bool):
    # Open advanced options
    page=page

    page.wait_for_selector("#toggle-button").click()
    if game_title:
        game_title_input=page.wait_for_selector(
            ".ytcp-form-gaming > "
            "ytcp-dropdown-trigger:nth-child(1) > "
            ":nth-child(2) > div:nth-child(3) > input:nth-child(3)"
        )
        game_title_input.send_keys(game_title)

        # Select first item in game drop down
        page.wait_for_selector("#text-item-2").click()

    # WebDriverWait(page, 20).until(EC.element_to_be_clickable(
        # ("VIDEO_MADE_FOR_KIDS_MFK" if made_for_kids else "VIDEO_MADE_FOR_KIDS_NOT_MFK")
    # )).click()

def _set_endcard(self):
    page=page

    # Add endscreen
    page.wait_for_selector("#endscreens-button").click()
    sleep(5)

    for i in range(1, 11):
        try:
            # Select endcard type from last video or first suggestion if no prev. video
            page.wait_for_selector(
                "div.card:nth-child(1)").click()
            break
        except:
            logging.warning(
                f"Couldn't find endcard button. Retry in 5s! ({i}/10)")
            sleep(5)

    page.is_visible("save-button").click()
# def close(self):
#    page.close()
#    page.quit()

#     self.log.debug("Closed Firefox")

def remove_unwatched_videos(self,page, remove_copyrighted, remove_unwatched_views):
    try:
        page.goto(YOUTUBE_URL)
        sleep(USER_WAITING_TIME)

        # set english as language
        self.__set_channel_language_english()

        page.get("https://studio.youtube.com/")
        sleep(USER_WAITING_TIME)
        page.wait_for_selector("menu-paper-icon-item-1").click()
        sleep(USER_WAITING_TIME)

        if self.__is_videos_available():
            return True

        page.wait_for_selector(
            "#page-size .ytcp-text-dropdown-trigger").click()
        sleep(USER_WAITING_TIME)
        # clock 50 items per page
        pagination_sizes=page.wait_for_selector(
            "#select-menu-for-page-size #dialog .paper-item")
        pagination_sizes[2].click()
        sleep(USER_WAITING_TIME)

        # filter to delete only copyrighted videos
        if remove_copyrighted:
            page.wait_for_selector("filter-icon").click()
            sleep(USER_WAITING_TIME)
            page.wait_for_selector(
                "ytcp-text-menu#menu tp-yt-paper-dialog tp-yt-paper-listbox paper-item#text-item-1 ytcp-ve div").click()
            sleep(USER_WAITING_TIME)

        # filter to delete videos with views lower than 100
        if remove_unwatched_views:
            views_no="100000"
            page.wait_for_selector("filter-icon").click()
            sleep(USER_WAITING_TIME)
            page.wait_for_selector(
                "ytcp-text-menu#menu tp-yt-paper-dialog tp-yt-paper-listbox paper-item#text-item-5 ytcp-ve div").click()
            sleep(USER_WAITING_TIME)
            page.wait_for_selector(
                "//iron-input[@id='input-2']/input").click()
            sleep(USER_WAITING_TIME)
            page.wait_for_selector(
                "//iron-input[@id='input-2']/input").clear()
            sleep(USER_WAITING_TIME)
            page.wait_for_selector(
                "//iron-input[@id='input-2']/input").send_keys(views_no)
            sleep(USER_WAITING_TIME)
            page.wait_for_selector(
                "//input[@type='text']").click()
            sleep(USER_WAITING_TIME)
            page.wait_for_selector(
                "//tp-yt-paper-listbox[@id='operator-list']/paper-item[2]").click()
            sleep(USER_WAITING_TIME)
            page.wait_for_selector(
                "//ytcp-button[@id='apply-button']/div").click()
            sleep(USER_WAITING_TIME)

        return self.__remove_unwatched_videos()
    except Exception as e:
        print(e)
        return False

def __is_videos_available(self,page):
    # if there are no videos to be deleted, this element should be visible
    # if not visible throw error, and proceed to delete more videos
    try:
        page.wait_for_selector(
            "//ytcp-video-section-content[@id='video-list']/div/div[2]/div")
        # return True, there are no more video to be deleted
        return True
    except:
        return False

def __write_in_field(self, field, string, select_all=False):
    field.click()

    sleep(USER_WAITING_TIME)
    if select_all:
        if self.is_mac:
            field.send_keys(Keys.COMMAND + 'a')
        else:
            field.send_keys(Keys.CONTROL + 'a')
        sleep(USER_WAITING_TIME)
    field.send_keys(string)

#     def __set_scheduler(self, publish_date):
#         # Set upload time
#         action=ActionChains(self.page)
#         schedule_radio_button=page.wait_for_selector("schedule-radio-button")

#         action.move_to_element(schedule_radio_button)
#         action.click(schedule_radio_button).perform()
#         self.log.debug('Set delevery to {}'.format("schedule"))
#         sleep(.33)

#         # Set close action
#         action_close=ActionChains(self.page)
#         action_close.send_keys(Keys.ESCAPE)

#         # date picker
#         action_datepicker=ActionChains(self.page)
#         datepicker_trigger=page.wait_for_selector("datepicker-trigger")

#         action_datepicker.move_to_element(datepicker_trigger)
#         action_datepicker.click(datepicker_trigger).perform()
#         sleep(.33)

#         date_string=publish_date.strftime("%d.%m.%Y")
#         date_input=page.wait_for_selector(
#             '//ytcp-date-picker/tp-yt-paper-dialog//iron-input/input')
#         # date_input.clear()
#         # # Transform date into required format: Mar 19, 2021
#         # date_input.send_keys(publish_date.strftime("%b %d, %Y"))
#         # date_input.send_keys(Keys.RETURN)

#         self.__write_in_field(date_input, date_string, True)
#         self.log.debug('Set schedule date to {}'.format(date_string))

#         action_close.perform()
#         sleep(.33)

#         # time picker
#         action_timepicker=ActionChains(self.page)
#         time_of_day_trigger=page.wait_for_selector("time-of-day-trigger")

#         action_timepicker.move_to_element(time_of_day_trigger)
#         action_timepicker.click(time_of_day_trigger).perform()
#         sleep(.33)

#         time_dto=(publish_date - timedelta(
#             minutes=publish_date.minute % 15,
#             seconds=publish_date.second,
#             microseconds=publish_date.microsecond))
#         time_string=time_dto.strftime("%H:%M")

#         time_container=page.wait_for_selector(
#             '//ytcp-time-of-day-picker//*[@id="dialog"]')
#         time_item=page.wait_for_selector(
#             '//ytcp-time-of-day-picker//tp-yt-paper-item[text() = "{}"]'.format(time_string))

#         self.log.debug('Set schedule date to {}'.format(time_string))
#        page.execute_script(
#             "arguments[0].scrollTop = arguments[1].offsetTop; ", time_container, time_item)

#         time_item.click()

#         action_close.perform()
#         sleep(.33)

#     def __remove_unwatched_videos(self):
#         DELETE_WAIT_TIME=60 * 2

#         # check if videos deletion process has finished
#         # if not visible throw error, and proceed to delete more videos
#         try:
#            page.wait_for_selector(
#                 "//div[@id='header']/div/span[2]")
#             # wait for the videos to be deleted and try delete videos after
#             sleep(DELETE_WAIT_TIME)
#             return self.__remove_unwatched_videos()
#         except:
#             pass

#         if self.__is_videos_available():
#             return True

#        page.wait_for_selector("checkbox-container").click()
#         sleep(USER_WAITING_TIME)
#        page.wait_for_selector(".ytcp-bulk-actions .toolbar .ytcp-select .ytcp-text-dropdown-trigger .ytcp-dropdown-trigger .right-container .ytcp-dropdown-trigger").click()
#         sleep(USER_WAITING_TIME)
#        page.wait_for_selector(
#             "#select-menu-for-additional-action-options #dialog #paper-list #text-item-1").click()
#         sleep(USER_WAITING_TIME)
#        page.wait_for_selector(
#             "#dialog-content-confirm-checkboxes #confirm-checkbox #checkbox-container").click()
#         sleep(USER_WAITING_TIME)
#        page.wait_for_selector(
#             ".ytcp-confirmation-dialog #dialog-buttons #confirm-button").click()
#         # wait 5 minutes for the videos to be deleted
#         sleep(DELETE_WAIT_TIME)

#         return self.__remove_unwatched_videos()


def waitfordone(page):

    # wait until video uploads
    # uploading progress text contains ": " - Timp ramas/Remaining time: 3 minutes.
    # we wait until ': ' is removed, so we know the text has changed and video has entered processing stage
    uploading_progress_text=page.locator(UPLOADING_PROGRESS_SELECTOR).text_content()
    while ': ' in uploading_progress_text:
        sleep(5)
        page.locator( UPLOADING_PROGRESS_SELECTOR).text_content()

def uploadTikTok(username, tiktok, deletionStatus, file):
    regex = re.compile('[0-9]{17}')
    regexA = re.compile('[0-9]{18}')
    regexB = re.compile('[0-9]{19}')
    regexC = re.compile('[0-9]{8}')
    regexD = re.compile('[0-9]{9}')
    if os.path.isdir(tiktok):
        if (
            regex.match(str(tiktok))
            or (regexA.match(str(tiktok)))
            or (regexB.match(str(tiktok)))
            or (regexC.match(str(tiktok)))
            or (regexD.match(str(tiktok)))
        ):  # TODO: use or regex with "|" instead of this
            item = get_item('tiktok-' + tiktok)
            if username is None:
                if file is not None:
                    file.write(str(tiktok))
                    file.write('\n')
                return None
            item.upload(
                './' + tiktok + '/',
                verbose=True,
                checksum=True,
                delete=deletionStatus,
                metadata=dict(
                    collection='opensource_media',
                    subject='tiktok',
                    creator=username,
                    title='TikTok Video by ' + username,
                    originalurl='https://www.tiktok.com/@' + username + '/video/' + tiktok,
                    scanner='TikUp ' + getVersion(),
                ),
                retries=9001,
                retries_sleep=60,
            )
            if deletionStatus:
                os.rmdir(tiktok)
            print()
            print('Uploaded to https://archive.org/details/tiktok-' + tiktok)
            print()
            if file is not None:
                file.write(str(tiktok))
                file.write('\n')




from setuptools import setup, find_packages
import re


with open("README.md", "r",encoding="utf-8") as f:
    long_description = f.read()


with open("ytb_up/__init__.py") as f:
    version = re.search(
        r"""^__version__\s*=\s*['"]([^\'"]*)['"]""", f.read(), re.MULTILINE
    ).group(1)


setup(
    name="ytb-up",
    version=version,
    author="wanghaisheng",
    author_email="edwin.uestc@gmail.com",
    description="auto video machine,Upload videos to YouTube using geckodriver, Firefox profiles and Selenium/playwright.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    url="https://github.com/wanghaisheng/youtube-auto-upload",
    download_url="https://github.com/wanghaisheng/youtube-auto-upload/tarball/v" + version,
    packages=["ytb_up"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=["playwright"],
    python_requires=">=3.6",
)



import json
from typing import Dict, List

""" Login module """


def domain_to_url(domain: str) -> str:
    """ Converts a (partial) domain to valid URL """
    if domain.startswith("."):
        domain = "www" + domain
    return "http://" + domain

def login_using_cookie_file(self, cookie_file: str,page):
    """Restore auth cookies from a file. Does not guarantee that the user is logged in afterwards.
    Visits the domains specified in the cookies to set them, the previous page is not restored."""
    domain_cookies: Dict[str, List[object]] = {}
    # cookie_file=r'D:\Download\audio-visual\make-reddit-video\auddit\assets\cookies\aww.json'
    with open(cookie_file) as file:
        cookies: List = json.load(file)
        # Sort cookies by domain, because we need to visit to domain to add cookies
        for cookie in cookies:
            try:
                domain_cookies[cookie["domain"]].append(cookie)
            except KeyError:
                domain_cookies[cookie["domain"]] = [cookie]
    # print(str(domain_cookies).replace(",", ",\n"))


    for domain, cookies in domain_cookies.items():
        # while True:
        page.goto(domain_to_url(domain + "/robots.txt"))
        for cookie in cookies:
            cookie.pop("sameSite", None)  # Attribute should be available in Selenium >4
            cookie.pop("storeId", None)  # Firefox container attribute
        print('add cookies')
    self.context.add_cookies(cookies)
def confirm_logged_in(page) -> bool:
    """ Confirm that the user is logged in. The browser needs to be navigated to a YouTube page. """
    try:
        print(page.locator("yt-img-shadow.ytd-topbar-menu-button-renderer > img:nth-child(1)"))
        page.locator("yt-img-shadow.ytd-topbar-menu-button-renderer > img:nth-child(1)")

        # WebDriverWait(page, 10).until(EC.element_to_be_clickable("avatar-btn")))
        return True
    except TimeoutError:
        return False
def confirm_logged_in(page) -> bool:
    """ Confirm that the user is logged in. The browser needs to be navigated to a YouTube page. """
    try:
        print(page.locator("yt-img-shadow.ytd-topbar-menu-button-renderer > img:nth-child(1)"))
        page.locator("yt-img-shadow.ytd-topbar-menu-button-renderer > img:nth-child(1)")

        # WebDriverWait(page, 10).until(EC.element_to_be_clickable("avatar-btn")))
        return True
    except TimeoutError:
        return False



# Darin McLain
# CS595 - Web Security - ODU - Spring 2021
# Assignment 3
# 
# Script to process header files and parse out cookie information
# Output results in table format to cookies.txt

import requests
import json
import os
import re

#https://stackoverflow.com/questions/32281041/converting-cookie-string-into-python-dict
def convertcookie2valid(cookie: str, parent_domain: str):
    items = cookie.split(';')
    SameSite = HttpOnly = Secure = Domain = Path = Expires = Comment = MaxAge = CookieName = CookieValue = Size = Sessionkey = Version = Priority = None
    CookieName = CookieValue = None
    idx = len(items) - 1
    while idx >= 0:
        item = items[idx].strip()
        idx -= 1
        if not item:
            continue
        SameSiteMatched = re.match(r'^SameSite(.*)?', item, re.I)
        HttpOnlyMatched = SameSiteMatched or re.match(r'^HttpOnly(.*)$', item, re.I)
        SecureMatched = HttpOnlyMatched or re.match(r'^Secure(.*)$', item, re.I)
        DomainMatched = SecureMatched or re.match(r'^Domain(.*)?', item, re.I)
        PathMatched = DomainMatched or re.match(r'^Path(.*)?', item, re.I)
        ExpiresMatched = PathMatched or re.match(r'^Expires(.*)?', item, re.I)
        CommentMatched = ExpiresMatched or re.match(r'^Comment(.*)?', item, re.I)
        MaxAgeMatched = ExpiresMatched or re.match(r'^Max-Age=(.*)?', item, re.I)
        VersionMatched = MaxAgeMatched or re.match(r'^Version=(.*)?', item, re.I)
        PriorityMatched = VersionMatched or re.match(r'^priority=(.*)?', item, re.I)
        matched = SameSiteMatched or HttpOnlyMatched or SecureMatched or DomainMatched or PathMatched or ExpiresMatched or CommentMatched or MaxAgeMatched or VersionMatched or PriorityMatched
        if matched:
            val = matched.groups(0)[0].lstrip('=')
            if matched == SameSiteMatched:
                SameSite = val if val.lower() in ['strict', 'lax', 'none'] else None
            elif matched == HttpOnlyMatched:
                HttpOnly = True
            elif matched == SecureMatched:
                Secure = True
            elif matched == DomainMatched:
                Domain = val
            elif matched == PathMatched:
                Path = val
            elif matched == PathMatched:
                Path = val
            elif matched == ExpiresMatched:
                Expires = val
            elif matched == CommentMatched:
                Comment = val
            elif matched == MaxAgeMatched:
                MaxAge = val
            elif matched == VersionMatched:
                Version = val
            elif matched == PriorityMatched:
                Priority = val
        else:
            CookieMatched = re.match(r'^(.[^=]*)=(.*)?', item, re.I)
            if CookieMatched:
                CookieName, CookieValue = CookieMatched.groups(0)

    Sessionkey = True if not Expires else False
    Size = (len(CookieName) if CookieName else 0) + (len(CookieValue) if CookieValue else 0)

    Domain = parent_domain if CookieName and not Domain else Domain
    Path = '/' if CookieName and not Path else Path
    Priority = 'Medium' if CookieName and not Priority else Priority.title() if Priority else 'Medium'

    Cookie = {
        CookieName: CookieValue,
        'Domain': Domain,
        'Path': Path,
        'Expires': Expires,
        'Comment': Comment,
        'MaxAge': MaxAge,
        'SameSite': SameSite,
        'HttpOnly': HttpOnly,
        'Secure': Secure,
        'Size': Size,
        'Sessionkey': Sessionkey,
        'Version': Version,
        'Priority': Priority
    }
    return Cookie if CookieName else None

# loop through all of the headers files
parentDirectory = os.path.abspath(os.path.join(os.getcwd(), os.pardir))
headersDirectory = os.path.join(parentDirectory, "3", "assets", "headers")
assetsDirectory = os.path.join(parentDirectory, "3", "assets")
setCookie = "set-cookie:"
results = {}


for file in os.listdir(headersDirectory):
    cookies = redirects = httpOnly = secure = samesite = strict = lax = none = path = slash = 0 # reset counters
    statuscode = None

    with open(os.path.join(headersDirectory, file)) as headerfile:
        url = file[:-4] # get url from txt file, strip out .txt
        result = {}
        
        for line in headerfile.readlines():
            # Process cookies
            if setCookie in line.lower():
                cookies += 1 # count the cookie
                line = line[len(setCookie) + 1:] # strip out Set-Cookie: 
                cookie = Robotcookie(line, "") # create object out of cookie for easy calculations
                
                # check keys for stats
                if (cookie['HttpOnly']):
                    httpOnly += 1
                if (cookie['Secure']):
                    secure += 1
                
                if (cookie['SameSite'] != None):
                    samesite += 1

                    if (str(cookie['SameSite']).lower() == "strict"):
                        strict += 1
                    elif (str(cookie['SameSite']).lower() == "lax"):
                        lax += 1
                    else:
                        none += 1

                if (cookie['Path'] != None):
                    path += 1
                    if (cookie['Path'] != "/"):
                        slash += 1

            if (line.startswith("HTTP/")):
                if (statuscode != None):
                    redirects += 1
                statuscode = line.rstrip()

        result = {
            'Cookies': cookies,
            'StatusCode': statuscode,
            'Redirects': redirects,
            'HttpOnly': httpOnly,
            'Secure': secure,
            'SameSite': samesite,
            'Strict': strict,
            'Lax': lax,
            'None': none,
            'Path': path,
            'NotSlash': slash
        }

        results[url] = result

with open(os.path.join(assetsDirectory, "cookies.json"), "w") as outfile:  # https://www.geeksforgeeks.org/how-to-convert-python-dictionary-to-json/
    json.dump(results, outfile, separators=(',', ': '), indent=4)


import json
from .constants import *
from .logging import Log
from .exceptions import *
from .utils import *
import os
from .login import *
from datetime import timedelta, date

from time import sleep
import logging
import re
from playwright.sync_api import *
import pickle
PROTECTED_FILES = ["processed.mp4", "VideosSaveHere.txt"]

class TiktokUpload:
    def __init__(
        self,
        root_profile_directory: str,
        proxy_option: str = "",
        timeout: int = 3,
        headless: bool = True,
        debug: bool = True,
        ytb_username:str ="",
        ytb_password:str ="",
        ytb_cookies:str="",
        tiktok_cookies:str="",
        tiktok_username:str="",
        tiktok_password:str=""

    ) -> None:
        self._playwright = self._start_playwright()
            #     browser = p.chromium.launch()

        PROXY_SOCKS5 = "socks5://127.0.0.1:1080"

        if not headless:
            headless = True
        if proxy_option == "":
            print('start web page without proxy')

            browserLaunchOptionDict = {
                "headless": headless,
                # "executable_path": executable_path,
                "timeout": 30000
            }

            if not root_profile_directory:
                self.browser = self._start_browser("firefox", **browserLaunchOptionDict)
            else:
                self.browser = self._start_persistent_browser(
                    "firefox", user_data_dir=root_profile_directory, **browserLaunchOptionDict
                )
        # Open new page
            self.context = self.browser.new_context()
        else:
            print('start web page with proxy')

            browserLaunchOptionDict = {
                "headless": False,
                "proxy": {
                    "server": PROXY_SOCKS5,
                },

                # timeout <float> Maximum time in milliseconds to wait for the browser instance to start. Defaults to 30000 (30 seconds). Pass 0 to disable timeout.#
                "timeout": 30000
            }

            if not root_profile_directory:
                self.browser = self._start_browser("firefox", **browserLaunchOptionDict)
            else:
                self.browser = self._start_persistent_browser(
                    "firefox", user_data_dir=root_profile_directory, **browserLaunchOptionDict
                )
        # Open new page
            self.context = self.browser.new_context()
        self.timeout = timeout
        self.log = Log(debug)
        self.ytb_username=ytb_username
        self.ytb_password=ytb_password
        self.ytb_cookies = ytb_cookies
        self.tiktok_username=tiktok_username
        self.tiktok_cookies=tiktok_cookies
        self.tiktok_password=tiktok_password
        self.log.debug("Firefox is now running")

    # Class used to upload video.

    def upload(
        self,
        videopath: str,
        title: str = "",
        description: str = "",
        thumbnail: str = "",
        publishpolicy: str = 0,
        # mode a:release_offset exist,publish_data exist will take date value as a starting date to schedule videos
        # mode b:release_offset not exist, publishdate exist , schedule to this specific date
        # mode c:release_offset not exist, publishdate not exist,daily count to increment schedule from tomorrow
        # mode d: offset exist, publish date not exist, daily count to increment with specific offset schedule from tomorrow
        release_offset: str = '0-1',
        publish_date: datetime = datetime(
            date.today().year,  date.today().month,  date.today().day, 10, 15),
        tags: list = [],
        process100:int =0
    ) -> Tuple[bool, Optional[str]]:
        """Uploads a video to YouTube.
        Returns if the video was uploaded and the video id.
        """
        self.page = self.context.new_page()
        print('============tags',tags)
        if not videopath:
            raise FileNotFoundError(f'Could not find file with path: "{videopath}"')

    def tiktokUpload(self):
        page=self.page
        # Cookies loaded here.

        if self.CHANNEL_COOKIES and not self.CHANNEL_COOKIES == '':
            print('cookies existing', self.CHANNEL_COOKIES)
            self.cookies = self.load_cookies(self.tiktok_cookies)

       
            # login_using_cookie_file(self,self.CHANNEL_COOKIES,page)         
            page.goto(TIKTOK_URL)

            page.reload()
        else:
            self.log.info('Please sign in and then press enter')
            # input()
            page.goto(TIKTOK_URL)
            # Interact with login form
            browser_context = self.browser.new_context(
                ignore_https_errors=True)
            browser_context.clear_cookies()
            # page.click('text=Login')
            # page.fill('input[name="login"]', USERNAME)
            # page.fill('input[name="password"]', PASSWORD)
            # page.click('text=Submit')
            sleep(USER_WAITING_TIME)
            storage = browser_context.storage_state(path=self.tiktok_cookies)
            self.context = browser_context

        islogin = confirm_logged_in(page)
        print('checking login status', islogin)

        if not islogin:
            print('try to load cookie files')
            self.cookies = self.load_cookies(self.tiktok_cookies)
            print('success load cookie files')
            page.goto(TIKTOK_URL)
            print('start to check login status')
            islogin = confirm_logged_in(page)
            
        print('start change locale to english')

        set_channel_language_english(page)
        print('finish change locale to english')
        page.goto(YOUTUBE_UPLOAD_URL)
        # sleep(self.timeout)
        self.log.debug(f'found to YouTube account check')

        if page.locator("#select-files-button")is None and page.locator("//*[@id='dialog-title']"):
            print('try to input credentials')
            verify(self,page)
        #confirm-button > div:nth-child(2)


        self.log.debug(f'Trying to upload "{videopath}" to YouTube...')
        if os.path.exists(get_path(videopath)):
            page.locator(
                INPUT_FILE_VIDEO)
            page.set_input_files(INPUT_FILE_VIDEO, get_path(videopath))
        else:
            if os.path.exists(videopath.encode('utf-8')):
                print('videopath found', videopath)
                page.locator(
                    INPUT_FILE_VIDEO)
                page.set_input_files(INPUT_FILE_VIDEO, videopath.encode('utf-8'))
        sleep(self.timeout)
        self.log.debug("Found YouTube upload Dialog Modal")

        # page = page.locator(UPLOAD_DIALOG_MODAL)


        # # Catch max uploads/day limit errors
        # if page.get_attribute(NEXT_BUTTON, 'hidden') == 'true':
        #     error_short_by_xpath=page.locator(ERROR_SHORT_XPATH)
        #     # print(f"ERROR: {error_short_by_xpath.text} {self.cookie_working_dir}")
        #     return False

        self.log.debug(f'Trying to set "{title}" as title...')


        # get file name (default) title
        # title=title if title else page.locator(TEXTBOX).text_content()
        # print(title)
        sleep(self.timeout)
        if len(title) > TITLE_COUNTER:
            print(f"Title was not set due to exceeding the maximum allowed characters ({len(title)}/{TITLE_COUNTER})")
            title=title[:TITLE_COUNTER-1]

                # TITLE
        print('click title field to input')
        page.locator(TEXTBOX).click()
        print('clear existing title')
        page.keyboard.press("Backspace")
        page.keyboard.press("Control+KeyA")
        page.keyboard.press("Delete")
        print('filling new  title')

        page.keyboard.type(title)

        self.log.debug(f'Trying to set "{title}" as description...')

        if description:
            if len(description) > DESCRIPTION_COUNTER:
                print(
                    f"Description was not set due to exceeding the maximum allowed characters ({len(description)}/{DESCRIPTION_COUNTER})"
                )
                description=description[:4888]

            self.log.debug(f'Trying to set "{description}" as description...')
            print('click description field to input')
            page.locator(DESCRIPTION_CONTAINER).click()
            print('clear existing description')
            page.keyboard.press("Backspace")
            page.keyboard.press("Control+KeyA")
            page.keyboard.press("Delete")
            print('filling new  description')

            page.keyboard.type(description)


        if thumbnail:
            self.log.debug(f'Trying to set "{thumbnail}" as thumbnail...')
            if os.path.exists(get_path(thumbnail)):
                page.locator(
                    INPUT_FILE_THUMBNAIL).set_input_files(get_path(thumbnail))
            else:
                if os.path.exists(thumbnail.encode('utf-8')):
                    print('thumbnail found', thumbnail)
                    page.locator(INPUT_FILE_THUMBNAIL).set_input_files(
                        thumbnail.encode('utf-8'))
            sleep(self.timeout)

        self.log.debug('Trying to set video to "Not made for kids"...')
        # kids_section=page.locator(NOT_MADE_FOR_KIDS_LABEL)
        page.locator(RADIO_LABEL).click()
        sleep(self.timeout)
        print('not made for kids done')
        if tags is None or tags =="" or len(tags)==0:
            pass
        else:
            print('tags you give',tags)
            if type(tags) == list:
                tags=",".join(str(tag) for tag in tags)
                tags=tags[:500]
            else:
                tags=tags
            print('overwrite prefined channel tags',tags)
            if len(tags) > TAGS_COUNTER:
                print(f"Tags were not set due to exceeding the maximum allowed characters ({len(tags)}/{TAGS_COUNTER})")
                tags=tags[:TAGS_COUNTER]
            print('click show more button')
            sleep(self.timeout)
            page.locator(MORE_OPTIONS_CONTAINER).click()

            self.log.debug(f'Trying to set "{tags}" as tags...')
            page.locator(TAGS_CONTAINER).locator(TEXT_INPUT).click()
            print('clear existing tags')
            page.keyboard.press("Backspace")
            page.keyboard.press("Control+KeyA")
            page.keyboard.press("Delete")
            print('filling new  tags')
            page.keyboard.type(tags)



    # Method to check file is valid.
    def checkFileExtensionValid(self):
        if self.userRequest["dir"].endswith('.mp4'):
            pass
        else:
            self.bot.close()
            exit(f"File: {self.userRequest['dir']} has wrong file extension.")


    # This gets the hashtags from file and adds them to the website input
    def addCaptions(self, hashtag_file=None):
        if not hashtag_file:
            caption_elem = self.webbot.getCaptionElem()
            for hashtag in self.IO.getHashTagsFromFile():
                caption_elem.send_keys(hashtag)

    def clearCaptions(self):
        caption_elem = self.webbot.getCaptionElem()
        caption_elem.send_keys("")

    def inputScheduler(self, schdate, schtime):
        # In charge of selecting scheduler in the input.
        utils.randomTimeQuery()
        self.webbot.selectScheduleToggle()


    # This is in charge of adding the video into tiktok input element.
    def inputVideo(self, startTime=0, endTime=0):
        try:
            file_input_element = self.webbot.getVideoUploadInput()
        except Exception as e:
            print("Major error, cannot find the upload button, please update getVideoUploadInput() in Bot.py")
            print(f"Actual Error: {e}")
            file_input_element = ""
            exit()
        # Check if file has correct .mp4 extension, else throw error.
        self.video = Video(self.userRequest["dir"], self.userRequest["vidTxt"], self.userPreference)
        print(f"startTime: {startTime}, endTime: {endTime}")
        if startTime != 0 and endTime != 0 or endTime != 0:
            print(f"Cropping Video timestamps: {startTime}, {endTime}")
            self.video.customCrop(startTime, endTime)
        # Crop first and then make video.

        self.video.createVideo()  # Link to video class method
        while not os.path.exists(self.video.dir):  # Wait for path to exist
            time.sleep(1)
        abs_path = os.path.join(os.getcwd(), self.video.dir)
        file_input_element.send_keys(abs_path)



    def directUpload(self, filename, private=False, test=False):
        if self.bot is None:
            self.bot = Browser().getBot()
            self.webbot = Bot(self.bot)
        self.bot.get(self.url)
        utils.randomTimeQuery()
        self.cookies = Cookies(self.bot)
        self.bot.refresh()

        try:
            file_input_element = self.webbot.getVideoUploadInput()
        except Exception as e:
            print(f"Error: {e}")
            print("Major error, cannot find the file upload button, please update getVideoUploadInput() in Bot.py")
            file_input_element = None
            exit()
        abs_path = os.path.join(os.getcwd(), filename)
        try:
            file_input_element.send_keys(abs_path)
        except StaleElementReferenceException as e:
            try:
                self.bot.implicitly_wait(5)
                file_input_element = self.webbot.getVideoUploadInput()
                file_input_element.send_keys(abs_path)
            except Exception as e:
                print("Major error, cannot find the file upload button, please update getVideoUploadInput() in Bot.py")
                exit()


        # We need to wait until it is uploaded and then clear input.

        self.addCaptions()
        utils.randomTimeQuery()
        if private:
            self.webbot.selectPrivateRadio()  # private video selection
            utils.randomTimeQuery()
        else:
            """
            self.webbot.selectPublicRadio()  # public video selection
            utils.randomTimeQuery()
            """
            pass
        if not test:

            self.webbot.uploadButtonClick()  # upload button
        input("Press any button to exit")


    """
    Returns TikTok cookies
    """

    def load_cookies(self,cookies):
        self.context.clear_cookies()

        self.context.add_cookies(
            json.load(
                open(
                    cookies, 
                    'r'
                )
            )
        )             

    """
    Remove unnecessary fields from cookies
    """
    def strip_cookies(cookies):

        stripped_cookies = []
        for cookie in cookies:

            stripped_cookie = {
                'name' : cookie['name'],
                'value' : cookie['value']
            }

            stripped_cookies.append(stripped_cookie)

        return stripped_cookies

    """
    Save the TikTok cookies locally
    """
    def save_tiktok_cookies(new_cookies):

        stripped_cookies = strip_cookies(new_cookies)
        cookies_file_name = "tiktok_cookies.pkl"

        with open(cookies_file_name, 'wb') as cookies_file:
            pickle.dump(stripped_cookies, cookies_file)